$(document).ready(async function () {

  $('#tutorial').on('click', function () {
    $(document).ready(async function () {
      const driver = new Driver({
        doneBtnText: 'Sair',
        closeBtnText: 'Fechar',
        nextBtnText: 'Próximo',
        prevBtnText: 'Anterior',
        allowClose: false
      });
      driver.defineSteps([{
        element: '#btnFiltros',
        popover: {
          title: 'Botão Filtros',
          description: 'Clique aqui para abrir os filtros e realizar uma consulta.',
          position: 'right'
        }
      },
      {
        element: '#btnImprimir',
        popover: {
          title: 'Botão Imprimir',
          description: 'Clique aqui para imprimir um relatório.',
          position: 'right'
        }
      },
      {
        element: '#divRelatorio',
        popover: {
          title: 'Tabela com os Dados',
          description: 'Aqui, após clicar em consultar ficam os dados filtrados.',
          position: 'bottom'
        }
      }
      ]);
      driver.start();
    });
  });

  const parametrosVenda = await BuscaParametrosVenda();
  const utilizaProcessoCaixaCego = parametrosVenda[64] == '1';

  $('#txtMoeda').autocompleta(1, `JSON={ "tabela":"MOEDA", "camposSelect":[ "CODMOE CHAVE", "DESCRICAO"], "where": null}`);
  $('#txtCaixa').autocompleta(3, `JSON={ "tabela":"CAIXA_LOJA", "camposSelect":[ "CODIGO CHAVE", "DESCRICAO"], "where": null}`);

  $('#btn-Moeda').pesquisa('#txtMoeda', 'CODIGO', 'DESCRICAO', `/sisplan/funcoes/v1/pesquisa?JSON={ "tabela":"MOEDA", "camposSelect":[ "CODMOE CODIGO", "DESCRICAO"]}`, 'Pesquisa Moeda', 'MOEDA');
  $('#btn-Caixa').pesquisa('#txtCaixa', 'CODIGO', 'DESCRICAO', `/sisplan/funcoes/v1/pesquisa?JSON={ "tabela":"CAIXA_LOJA", "camposSelect":[ "CODIGO", "DESCRICAO"]}`, 'Pesquisa Caixa', 'CAIXA_LOJA');

  $('#btn-limpa-Moeda').on('click', function () {
    $('#txtMoeda').val('');
  });

  $('#btn-limpa-Caixa').on('click', function () {
    $('#txtCaixa').val('');
  });

  $('#btnLimparFiltros').on('click', async function () {
    $('#dtPeriodoDe').val('');
    $('#dtPeriodoAte').val('');
    $('#txtMoeda').val('');
    $('#txtCaixa').val('');
  });

  $('#btnFiltros').on('click', async function () {
    $('#modal-filtros').modal('show');
  });

  async function retornaFiltros() {
    const periodoDe = $('#dtPeriodoDe').val();
    const periodoAte = $('#dtPeriodoAte').val();
    const moeda = pegaChave('#txtMoeda');
    const caixa = pegaChave('#txtCaixa');

    const retorno =
      `PERIODODE=${periodoDe}` +
      `&PERIODOATE=${periodoAte}` +
      `&MOEDA=${moeda}` +
      `&CAIXA=${caixa}`;

    return retorno;
  }

  $('#btnImprimir').on('click', async function () {
    try {
      $.LoadingOverlay('show');
      try {
        const url = `/sisplan/impressao/v1/imprimirrelcaixa?`;
        const filtros = await retornaFiltros();
        const bImprimeAutomatico = await verificaImprimirAutomatico('RelCaixa');
        const arquivo = await GeraRelatorio(`${url}${filtros}`, 'GET', 100000, false, 'RelCaixa');
        if (arquivo != undefined) {
          if (!bImprimeAutomatico) {
            window.open(`${BASE_URI}/relatorios_api/pdf/${arquivo}`, '_blank');
          }
          await limparRelatorios();
        }

      } catch (error) {
        console.error(error);
      }
    } finally {
      $.LoadingOverlay('hide');
    }
  });

  $('#btnConsultar').on('click', async function () {
    try {
      if ((!validarData($('#dtPeriodoDe').val())) || (!validarData($('#dtPeriodoAte').val()))) {
        msgAlerta('Necessário informar um período válido.');
        return;
      }

      if (pegaChave('#txtCaixa') == '') {
        msgAlerta('Necessário informar um caixa.');
        return;
      }

      $.LoadingOverlay("show");
      const filtros = await retornaFiltros();
      var response = await requisicao('GET', '/sisplan/impressao/v1/consultarelcaixa?', filtros, null);

      if (!response) {
        $.LoadingOverlay("hide");
        return;
      }

      var jsonStr = await response.json();
      if (response.status != 200) {
        $.LoadingOverlay("hide");
        alert(jsonStr.mensagem);
        return;
      }

      var table = criaDataTable(jsonStr[0], jsonStr[1], jsonStr[2]);

      $('#tabelaVendas_wrapper').css("width", "100%");
      $('#modal-filtros').modal('hide');
      $.LoadingOverlay("hide");
    } catch (error) {
      console.log(error);
      $.LoadingOverlay("hide");
    }
  });

  async function criaDataTable(data, data2, data3) {
    let dataSetCols = [];
    let datasetCols2 = [];
    let datasetCols3 = [];

    dataSetCols.push({
      "data": "CODIGO"
    });
    dataSetCols.push({
      "data": "DESCRICAO"
    });
    dataSetCols.push({
      "data": "CONDICAO"
    });
    dataSetCols.push({
      "data": "SALDO_ANTERIOR"
    });
    dataSetCols.push({
      "data": "ENTRADAS"
    });
    dataSetCols.push({
      "data": "SAIDAS"
    });
    dataSetCols.push({
      "data": "SALDO_ATUAL"
    });

    if (utilizaProcessoCaixaCego) {
      dataSetCols.push({
        "data": "CONFERIDO"
      });
      dataSetCols.push({
        "data": "DIFERENCA"
      });
    }

    datasetCols2.push({
      "data": "CODMOE"
    });
    datasetCols2.push({
      "data": "DESCMOE"
    });
    datasetCols2.push({
      "data": "VALOR"
    });
    datasetCols2.push({
      "data": "JUROS"
    });
    datasetCols2.push({
      "data": "VALOR_ORIG"
    });
    datasetCols2.push({
      "data": "DT_VENCTO"
    });
    datasetCols2.push({
      "data": "DESCTIPO"
    });
    datasetCols2.push({
      "data": "DUPLICATA"
    });
    datasetCols2.push({
      "data": "DESCRICAO"
    });
    datasetCols2.push({
      "data": "ORDEM"
    });

    datasetCols3.push({
      "data": "VENDA",
      "title": "Venda"
    });
    datasetCols3.push({
      "data": "MOEDA",
      "title": "Moeda"
    });
    datasetCols3.push({
      "data": "DESC_MOEDA",
      "title": "Desc Moeda"
    });
    datasetCols3.push({
      "data": "CONDICAO",
      "title": "Condição"
    });
    datasetCols3.push({
      "data": "QUANTIDADE",
      "title": "Quantidade"
    });
    datasetCols3.push({
      "data": "VALOR_VENDA",
      "title": "Valor Venda"
    });
    datasetCols3.push({
      "data": "VALOR",
      "title": "Valor"
    });
    datasetCols3.push({
      "data": "DESCONTO",
      "title": "Desconto"
    });

    let table3 = $('#tabelaVendasGrid').DataTable({
      searching: false,
      paging: false,
      info: false,
      destroy: true,
      ordering: false,
      "language": {
        "sEmptyTable": "Nenhum registro encontrado",
        "sInfo": "_TOTAL_ registros",
        "sInfoEmpty": " 0 registros",
        "sInfoFiltered": "(Filtrados de _MAX_ registros)",
        "sInfoPostFix": "",
        "sInfoThousands": ".",
        "sLengthMenu": "_MENU_ resultados",
        "sLoadingRecords": "Carregando...",
        "sProcessing": "Processando...",
        "sZeroRecords": "Nenhum registro encontrado",
        "sSearch": "Pesquisar",
        "oPaginate": {
          "sNext": "Próximo",
          "sPrevious": "Anterior",
          "sFirst": "Primeiro",
          "sLast": "Último"
        },
        "oAria": {
          "sSortAscending": ": Ordenar colunas de forma ascendente",
          "sSortDescending": ": Ordenar colunas de forma descendente"
        }
      },
      "autoWidth": true,
      "data": data3,
      "columns": datasetCols3,
      "columnDefs": [{
        "render": function (data) {
          return parseFloat(data ?? 0).toLocaleString("pt-BR", {
            maximumFractionDigits: 2,
            minimumFractionDigits: 2
          })
        },
        "targets": [5, 6, 7],
      }]
    });

    const colunasNumericas = utilizaProcessoCaixaCego ? [3, 4, 5, 6] : [3, 4, 5, 6];

    let table = $('#tabelaVendas').DataTable({
      bSort: false,
      searching: false,
      paging: false,
      info: false,
      destroy: true,

      "language": {
        "sEmptyTable": "Nenhum registro encontrado",
        "sInfo": "_TOTAL_ registros",
        "sInfoEmpty": " 0 registros",
        "sInfoFiltered": "(Filtrados de _MAX_ registros)",
        "sInfoPostFix": "",
        "sInfoThousands": ".",
        "sLengthMenu": "_MENU_ resultados",
        "sLoadingRecords": "Carregando...",
        "sProcessing": "Processando...",
        "sZeroRecords": "Nenhum registro encontrado",
        "sSearch": "Pesquisar",
        "oPaginate": {
          "sNext": "Próximo",
          "sPrevious": "Anterior",
          "sFirst": "Primeiro",
          "sLast": "Último"
        },
        "oAria": {
          "sSortAscending": ": Ordenar colunas de forma ascendente",
          "sSortDescending": ": Ordenar colunas de forma descendente"
        }
      },
      "columnDefs": [{
        "render": function (data) {
          return parseFloat(data ?? 0).toLocaleString("pt-BR", {
            maximumFractionDigits: 2,
            minimumFractionDigits: 2
          })
        },
        "targets": colunasNumericas,
      }],
      "autoWidth": true,
      "data": data,
      "columns": dataSetCols,
    });

    $(function () {
      $('.txtDINHEIRO').maskMoney({
        decimal: '.',
        thousands: '',
        precision: 2
      });
    })

    let table2 = $('#tabelaManual').DataTable({
      bSort: false,
      searching: false,
      paging: false,
      info: false,
      destroy: true,
      "language": {
        "sEmptyTable": "Nenhum registro encontrado",
        "sInfo": "_TOTAL_ registros",
        "sInfoEmpty": " 0 registros",
        "sInfoFiltered": "(Filtrados de _MAX_ registros)",
        "sInfoPostFix": "",
        "sInfoThousands": ".",
        "sLengthMenu": "_MENU_ resultados",
        "sLoadingRecords": "Carregando...",
        "sProcessing": "Processando...",
        "sZeroRecords": "Nenhum registro encontrado",
        "sSearch": "Pesquisar",
        "oPaginate": {
          "sNext": "Próximo",
          "sPrevious": "Anterior",
          "sFirst": "Primeiro",
          "sLast": "Último"
        },
        "oAria": {
          "sSortAscending": ": Ordenar colunas de forma ascendente",
          "sSortDescending": ": Ordenar colunas de forma descendente"
        }
      },
      "autoWidth": true,
      "data": data2,
      "columns": datasetCols2,
      "columnDefs": [{
        "type": 'date-br',
        "render": function (data) {
          if (data != undefined) {
            x = data.slice(0, 10).split('-');
            return `${x[2]}/${x[1]}/${x[0]}`;
          } else {
            return ''
          }
        },
        "targets": 5,
      },
      {
        "render": function (data) {
          return parseFloat(data ?? 0).toLocaleString("pt-BR", {
            maximumFractionDigits: 2,
            minimumFractionDigits: 2
          })
        },
        "targets": [2, 3, 4],
      }
      ]
    });
  }

  function validarData(data) {
    try {
      const dia = parseInt(data.substring(8));
      const mes = parseInt(data.substring(5,7));
      const ano = parseInt(data.substring(0,4));
  
      switch (mes) {
        case 1: case 3: case 5: case 7:
        case 8: case 10: case 12:
          if (dia <= 31)
            return true;
          else
            return false;
          break;
        case 4: case 6:
        case 9: case 11:
          if (dia <= 30)
            return true;
          else
            return false;
          break;
        case 2:
          if ((ano % 400 == 0) || (ano % 4 == 0 && ano % 100 != 0))
            if (dia <= 29)
              return true;
            else
              return false;
          else
            if (dia <= 28)
              return true;
            else
              return false;
      }
    } catch {
      return false;
    }
  }
});